'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function unwrapExports (x) {
	return x && x.__esModule ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var utils = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.getServiceName = getServiceName;
var hasWindow = typeof window !== 'undefined';

/**
 * Given a full URL address, parse the pathname
 *
 * @param {String} address A URL
 * @return {String} The path within the given URL
 */
function getServiceName(obj) {
  var address = void 0;
  if (typeof obj === 'string') {
    address = obj;
  } else if ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' && typeof obj.address === 'string') {
    address = obj.address;
  } else {
    throw new Error('getServiceName must be called with a string url or with a SystemJS load object that has an address');
  }

  var windowPath = hasWindow ? window.location.pathname.substring(0, window.location.pathname.lastIndexOf('/') + 1) : '';

  // The service name might have a path in it!
  // For example, you might `import a from 'service/a/path.js!sofe';`
  // In that case, we want `getServiceName` to return just `'sesrvice'`;
  var urlParts = getRegex().exec(address);
  var servicePath = urlParts[5];

  var splits = servicePath.substring(windowPath.length).split('/');

  // There might be a leading `/`, if so `splits[0]` is empty and
  // we want to get the second element of the array.
  return removeBang(splits[0] || splits[1]);
}

function removeBang(url) {
  return url.indexOf('!') > -1 ? url.substring(0, url.indexOf('!')) : url;
}

function getRegex() {
  return (/^(([^:\/?#]+):)?(\/\/([^\/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?/g
  );
}
});

var registries = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getUrlFromRegistry = getUrlFromRegistry;
var NPM_CDN = 'https://unpkg.com';
var hasWindow = typeof window !== 'undefined';

function getRegistryUrl(config) {
  if (config && config.registry) return config.registry;else return NPM_CDN;
}

/**
 * Get the url from an npm registry.
 *
 * @param {String} service The name of the service that is being resolved
 * @param {Object} config The sofe config object
 * @return {Promise} A promise which resolves with the service url
 */
function getUrlFromRegistry(service, config) {
  return new Promise(function (resolve, reject) {
    var requestUrl = getRegistryUrl(config) + '/' + service + '/package.json';

    if (!hasWindow) {
      return resolve('file://' + __dirname + '/blank.js');
    }

    var xhr = new XMLHttpRequest();
    xhr.addEventListener('load', xhrLoaded);
    xhr.addEventListener('error', xhrFailed);
    xhr.open('GET', requestUrl);
    xhr.send();

    function xhrLoaded() {
      if (Number(xhr.status) >= 200 && Number(xhr.status) < 300) {
        var json = void 0;
        try {
          json = JSON.parse(xhr.responseText);
        } catch (ex) {
          xhrFailed();
          return;
        }
        if (json.sofe) {
          // The registry is a simple sofe registry
          resolve(json.sofe.url);
        } else {
          // The registry is an NPM registry
          var version = json.version;

          if (json.sofe && json.sofe.url) {
            resolve(json.sofe.url);
          } else {
            resolve(NPM_CDN + '/' + service);
          }
        }
      } else {
        xhrFailed();
      }
    }

    function xhrFailed() {
      reject(new Error('Invalid registry response for service: ' + service + '\nRequest:' + requestUrl));
    }
  });
}
});

var manifest = createCommonjsModule(function (module, exports) {
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.getManifest = getManifest;
exports.getAllManifests = getAllManifests;
exports.clearManifest = clearManifest;
var cachedRemoteManifestPromise = {};
var cachedManifests = {};
var hasWindow = typeof window !== 'undefined';

/**
 * Asynchronously find a manifest for resolving sofe services. The manifest is a simple
 * map where key values represent unique services and the values represent a URL
 * to load the service. If there is no available manifest, sofe will default to
 * querying npm for service resolution.
 *
 * @param {Object} config the sofe configuration object
 * @param {Array} visitedManifestUrls keep track of previously visited urls to detect circular dependencies
 * @param {String} parent if this is a recursively loaded manifest, who was the parent that loaded it?
 * @return {Promise} A promise which is resolved with a service manifest.
 */
function getManifest(config) {
  var visitedManifestUrls = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var parent = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  // Don't fetch the manifest twice
  if (config.manifestUrl && cachedRemoteManifestPromise[config.manifestUrl]) {
    return cachedRemoteManifestPromise[config.manifestUrl];
  }

  var promise = new Promise(function (resolve, reject) {
    var staticManifest = config.manifest || {};

    if (config.manifestUrl && hasWindow) {
      var xhrLoaded = function xhrLoaded() {
        if (Number(xhr.status) >= 200 && Number(xhr.status) < 300) {
          var json = void 0;
          try {
            json = JSON.parse(xhr.responseText);
          } catch (ex) {
            xhrFailed();
            return;
          }

          visitedManifestUrls.push(config.manifestUrl);

          if (json && json.sofe) {

            // Record manifest tree information
            if (parent) {
              cachedManifests[config.manifestUrl] = {
                manifest: json.sofe.manifest || {},
                parent: parent
              };

              if (!cachedManifests[parent]) {
                cachedManifests[parent] = {
                  manifest: staticManifest,
                  parent: 'static'
                };
              }
            } else {
              cachedManifests.static = {
                manifest: staticManifest,
                parent: null
              };

              if (!cachedManifests[config.manifestUrl]) {
                cachedManifests[config.manifestUrl] = {
                  manifest: json.sofe.manifest || {},
                  parent: 'static'
                };
              }
            }

            // First check if this manifest points to another manifest
            if (typeof json.sofe.manifestUrl === 'string') {
              if (visitedManifestUrls.indexOf(json.sofe.manifestUrl) >= 0) {
                reject(new Error('Cannot load manifest -- circular chain of sofe manifests, \'' + json.sofe.manifestUrl + '\' detected twice in chain.'));
              } else {
                getManifest(json.sofe, visitedManifestUrls, config.manifestUrl).then(function (chainedManifest) {
                  if (config.manifest === Object(config.manifest)) {
                    resolve(_extends({}, chainedManifest, config.manifest));
                  } else {
                    resolve(chainedManifest);
                  }
                }).catch(function (err) {
                  reject(err);
                });
              }
            }
            // Otherwise make sure this manifest actually is a manifest
            else if (json.sofe.manifest) {
                /* Ensure that the manifest is an object, without lodash. See
                 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object
                 * and http://stackoverflow.com/questions/8511281/check-if-a-variable-is-an-object-in-javascript
                */
                if (json.sofe.manifest === Object(json.sofe.manifest)) {

                  var cachedRemoteManifest = _extends({}, json.sofe.manifest, staticManifest);

                  if (hasWindow) {
                    window.sofe.cachedRemoteManifest = cachedRemoteManifest;
                  }

                  resolve(cachedRemoteManifest);
                } else {
                  reject(new Error('Invalid manifest JSON at \'' + config.manifestUrl + '\': the \'manifest\' property must be a plain object'));
                }
              } else {
                reject(new Error('Invalid manifest JSON at \'' + config.manifestUrl + '\': there must either be a \'sofe.manifest\' object or \'sofe.manifestUrl\' string'));
              }
          } else {
            reject(new Error('Invalid manifest JSON at \'' + config.manifestUrl + '\': a manifest must include a sofe attribute'));
          }
        } else {
          xhrFailed();
        }
      };

      var xhrFailed = function xhrFailed() {
        reject(new Error('Invalid manifest: must be parseable JSON'));
      };

      var xhr = new XMLHttpRequest();
      xhr.addEventListener('load', xhrLoaded);
      xhr.addEventListener('error', xhrFailed);
      xhr.open('GET', config.manifestUrl);
      xhr.send();
    } else {
      // Resolve with no manifest if there is no config.manifest or config.manifestUrl
      resolve(staticManifest);
    }
  });

  if (config.manifestUrl) {
    cachedRemoteManifestPromise[config.manifestUrl] = promise;
  }

  return promise;
}

function getAllManifests(config) {
  return new Promise(function (resolve, reject) {
    getManifest(config).then(function (manifest) {
      resolve({
        flat: _extends({}, manifest),
        all: _extends({}, cachedManifests)
      });
    }).catch(reject);
  });
}

function clearManifest() {
  cachedRemoteManifestPromise = window.sofe.cachedRemoteManifest = {};
  cachedManifests = {};
}
});

var middleware = createCommonjsModule(function (module, exports) {
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.stepMiddleware = stepMiddleware;
function stepMiddleware(middleware, load, action) {
  if (middleware.length === 0) return action(load, []);

  // If there are any undefined middleware, replace them with a noop
  middleware = middleware.map(function (f) {
    return f ? f : function (load, next) {
      next(load);
    };
  }).map(function toMiddleWareThatAlwaysCallNext(f) {
    if (f.length === 1) {
      return function (load, next) {
        f(load);
        next(load);
      };
    } else {
      return f;
    }
  });

  if (middleware.length === 1) {
    var next = middleware[0](load, function (load) {
      // neet to skip the current frame in case the next
      // middleware is immediately called.
      setTimeout(function () {
        return action(load, [next]);
      });
    });
    return;
  }

  // keep track of the next middleware state
  var newMiddleware = [];

  var last = function last() {
    var _middleware;

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var next = (_middleware = middleware)[middleware.length - 1].apply(_middleware, args.concat([function (load) {
      // neet to skip the current frame in case the next
      // middleware is immediately called.
      setTimeout(function () {
        return action(load, [].concat(newMiddleware, [next]));
      });
    }]));
  };

  var rest = middleware.slice(0, -1);

  var chain = rest.map(function (f, index) {
    return function () {
      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      if (index !== rest.length - 1) {
        var _next = f.apply(undefined, args.concat([function (load) {
          return setTimeout(function () {
            return chain[index + 1](load);
          });
        }]));
        newMiddleware.push(_next);
      } else {
        var _next2 = f.apply(undefined, args.concat([last]));
        newMiddleware.push(_next2);
      }
    };
  });

  chain[0](load);
}
});

var hooks = createCommonjsModule(function (module, exports) {
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetch = undefined;
exports.InvalidServiceName = InvalidServiceName;
exports.getServiceUrl = getServiceUrl;
exports.isOverride = isOverride;
exports.locate = locate;
exports.setMiddleWare = setMiddleWare;

var _utils = utils;

var _registries = registries;

var _manifest = manifest;

var _middleware = middleware;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var config = SystemJS.sofe || {};

var hasWindow = typeof window !== "undefined";

var allMiddleware = [];
var serviceMap = {};
var serviceOverrides = [];
var middlewareMap = {};
var middlewareId = 0;
var middlewareTracker = 0;

function InvalidServiceName() {}
InvalidServiceName.prototype = Object.create(Error.prototype);

function getServiceUrl(name) {
  if (!serviceMap[name]) {
    throw new InvalidServiceName("Service \"" + name + "\" has not been loaded, import the service before getting the service URL");
  }
  return serviceMap[name];
}

function isOverride(service) {
  return service ? serviceOverrides.indexOf(service) > -1 : !!serviceOverrides.length;
}

/**
 * Override SystemJS loader locate method
 * @param {Object} load Load object from System.js loader
 *        See more at: https://github.com/ModuleLoader/es6-module-loader/blob/v0.17.0/docs/loader-extensions.md
 *
 * @return {Promise} A promise which resolves with the service url
 */
function locate(load) {
  middlewareId++;
  var id = middlewareId;

  return new Promise(function (resolvePromise, reject) {
    (0, _middleware.stepMiddleware)(allMiddleware, load, function (load, newMiddleware) {
      function resolve(url) {
        (0, _middleware.stepMiddleware)(newMiddleware, url, function (newUrl, newMiddleware) {
          middlewareMap[id] = newMiddleware;
          middlewareTracker = id;
          serviceMap[service] = newUrl;
          resolvePromise(newUrl);
        });
      }

      var service = (0, _utils.getServiceName)(load.address);

      //first check session storage (since it is very transient)
      if (hasWindow && window.sessionStorage && window.sessionStorage.getItem("sofe:" + service)) {
        var url = window.sessionStorage.getItem("sofe:" + service);
        addServiceOverride(service);
        resolve(url);
      } else if (hasWindow && window.localStorage && window.localStorage.getItem("sofe:" + service)) {
        //otherwise check local storage (since it is less transient)
        var _url = window.localStorage.getItem("sofe:" + service);
        addServiceOverride(service);
        resolve(_url);
      } else {
        //otherwise check manifest
        (0, _manifest.getManifest)(config).then(function (manifest$$1) {
          // First try and resolve the service with the manifest,
          // otherwise resolve by requesting the registry
          if (manifest$$1 && manifest$$1[service]) {
            resolve(manifest$$1[service]);
          } else {
            (0, _registries.getUrlFromRegistry)(service, config).then(function (url) {
              resolve(url);
            }).catch(function (error) {
              reject(error);
            });
          }
        }).catch(function (error) {
          reject(error);
        });
      }
    });
  });
}

function setMiddleWare(middleware$$1) {
  allMiddleware = [].concat(_toConsumableArray(allMiddleware), _toConsumableArray(middleware$$1));
}

function fetch(load, _systemFetch) {
  return new Promise(function (resolve, reject) {
    var middleware$$1 = middlewareMap[middlewareTracker] || [];
    delete middlewareMap[middlewareTracker];
    var systemFetchAlreadyCalled = false;

    (0, _middleware.stepMiddleware)(middleware$$1, {
      systemFetch: function systemFetch(load) {
        systemFetchAlreadyCalled = true;
        return _systemFetch(load);
      },
      load: load
    }, function (load) {
      load = load.load ? load.load : load;

      if (systemFetchAlreadyCalled) {
        resolve(load);
      } else {
        resolve(_systemFetch(load));
      }
    });
  });
}

exports.fetch = fetch;
function addServiceOverride(service) {
  if (serviceOverrides.indexOf(service) === -1) {
    serviceOverrides.push(service);
  }
}

if (typeof window !== "undefined") {
  window.sofe = {
    clearCache: function clearCache() {
      serviceMap = {};
      allMiddleware = [];
      middlewareMap = {};
      serviceOverrides = [];
      (0, _manifest.clearManifest)();
    }
  };
}
});

var sofe = createCommonjsModule(function (module, exports) {
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getServiceName = exports.InvalidServiceName = exports.getServiceUrl = exports.isOverride = exports.fetch = exports.locate = undefined;
exports.getAllManifests = getAllManifests;
exports.applyMiddleware = applyMiddleware;

var _utils = utils;

Object.defineProperty(exports, "getServiceName", {
  enumerable: true,
  get: function get() {
    return _utils.getServiceName;
  }
});
exports.getManifest = getManifest;

var _hooks = hooks;

var _manifest = manifest;

function getAllManifests() {
  return (0, _manifest.getAllManifests)(System.sofe || {});
}

function applyMiddleware() {
  for (var _len = arguments.length, middleware = Array(_len), _key = 0; _key < _len; _key++) {
    middleware[_key] = arguments[_key];
  }

  var chain = middleware.map(function (middleware) {
    return middleware();
  });
  (0, _hooks.setMiddleWare)(chain);
}

exports.locate = _hooks.locate;
exports.fetch = _hooks.fetch;
exports.isOverride = _hooks.isOverride;
exports.getServiceUrl = _hooks.getServiceUrl;
exports.InvalidServiceName = _hooks.InvalidServiceName;
function getManifest(url) {
  if (typeof url !== "string") {
    throw new Error("sofe getManifest API must be called with a url string");
  }

  // Ensure that the manifest for this url has been retrieved
  return (0, _manifest.getManifest)({ manifestUrl: url }).then(function () {
    /* We don't want the merged combination of all of the chained manifests,
     * which is what _getManifest returns. Instead, we want *just* the manifest
     * exactly as it is found at the specified url.
     */
    return getAllManifests().then(function (manifests) {
      return manifests.all[url].manifest;
    });
  });
}

if (typeof localStorage !== "undefined" && !localStorage.getItem("disable-sofe-override-warning")) {
  var numOverrides = 0;
  for (var i = 0; i < localStorage.length; i++) {
    var key = localStorage.key(i);
    var serviceName = key.slice("sofe:".length);

    if (key.match(/sofe:(\S)+/g)) {
      console.info("There is a local storage sofe override for the service '" + serviceName + "' to url '" + localStorage.getItem(key) + "'");
      numOverrides++;
    }
  }
  if (numOverrides > 0) {
    console.info("Run localStorage.setItem('disable-sofe-override-warning', true) to turn off sofe override warnings");
  }
}

if (typeof sessionStorage !== "undefined" && !sessionStorage.getItem("disable-sofe-override-warning")) {
  var _numOverrides = 0;
  for (var _i = 0; _i < sessionStorage.length; _i++) {
    var _key2 = sessionStorage.key(_i);
    var _serviceName = _key2.slice("sofe:".length);

    if (_key2.match(/sofe:(\S)+/g)) {
      console.info("There is a session storage sofe override for the service '" + _serviceName + "' to url '" + sessionStorage.getItem(_key2) + "'");
      _numOverrides++;
    }
  }
  if (_numOverrides > 0) {
    console.info("Run sessionStorage.setItem('disable-sofe-override-warning', true) to turn off sofe override warnings");
  }
}
});

var sofe$1 = unwrapExports(sofe);
var getServiceName = sofe.getServiceName;
var InvalidServiceName = sofe.InvalidServiceName;
var getServiceUrl = sofe.getServiceUrl;
var isOverride = sofe.isOverride;
var fetch = sofe.fetch;
var locate = sofe.locate;
var getAllManifests = sofe.getAllManifests;
var applyMiddleware = sofe.applyMiddleware;
var getManifest = sofe.getManifest;

exports['default'] = sofe$1;
exports.getServiceName = getServiceName;
exports.InvalidServiceName = InvalidServiceName;
exports.getServiceUrl = getServiceUrl;
exports.isOverride = isOverride;
exports.fetch = fetch;
exports.locate = locate;
exports.getAllManifests = getAllManifests;
exports.applyMiddleware = applyMiddleware;
exports.getManifest = getManifest;
